// DateInterval.h  Datumsintervall-Berechner
//
// Doku: s. Header-File
//
// (c) 2003, oose.de GmbH
//////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DateInterval.h"

DateInterval::DateInterval(){
	day = 1;
	month = 1;
	year = MIN_YEAR;
	for (int i = 0; i < MAX_MONTH; i++){
		monthDays[i] = dayOfMonth[i];
	}
}

DateInterval::~DateInterval(){
}

bool DateInterval::isLeapYear(int year){
	bool leap = false;

	if (year > 0){				// nicht checkdate() aufrufen, sonst endlos rekursiv!
		if (0 == year % 4){
			leap = true;
			if (0 == year % 100){
				leap = false;
				if (0 == year % 400){
					leap = true;
				}
			}
		}
	}
	return leap;
}

bool DateInterval::checkDate(int day, int month, int year){
	if ((year < MIN_YEAR) || (year > MAX_YEAR)) return false;
	if ((month < 1) || (month > MAX_MONTH)) return false;
	if ((!isLeapYear(year)) || (LEAP_MONTH != month)){
		if ((day < 1) || (day > dayOfMonth[month-1])) return false;
	}
	else {
		if ((day < 1) || (day > LEAP_MONTH_DAYS)) return false;
	}

	return true;
}

void DateInterval::updateDaysOfMonth(){
	if (isLeapYear(getYear())){
		monthDays[LEAP_MONTH-1] = LEAP_MONTH_DAYS;
	}
	else{
		monthDays[LEAP_MONTH-1] = REG_MONTH_DAYS;
	}

}


bool DateInterval::reset(int day, int month, int year){
	bool ret = checkDate(day, month, year);
	if (true == ret){
		this->day = day;
		this->month = month;
		this->year = year;
	}
	updateDaysOfMonth();	// ggf. Monatstage an Schaltjahre anpassen
	return ret;
}


void DateInterval::nextDay(int &day, int &month, int &year){
	year = getYear();

	if (getDay() < monthDays[getMonth()-1]){
		day = getDay() + 1;
		month = getMonth();
	}
	else {
		day = 1;
		if (getMonth() < MAX_MONTH){
			month = getMonth()+1;
		}
		else{
			month = 1;
			year = getYear()+1;
		}
	}
}

void DateInterval::nextWeek(int &day, int &month, int &year){
	year = getYear();

	if (getDay() + WEEK_LEN < monthDays[getMonth()-1]){
		day = getDay()+WEEK_LEN;
		month = getMonth();
	}
	else{
		day = getDay()+WEEK_LEN-monthDays[getMonth()-1];
		if (getMonth() < MAX_MONTH){
			month = getMonth()+1;
		}
		else{
			month = 1;
			year = getYear()+1;
		}
	}
}

void DateInterval::nextMonth(int &day, int &month, int &year){
	int nextYear = getYear();
	int nextMon = getMonth() + 1;

	if (nextMon > MAX_MONTH){
		nextMon = 1;
		nextYear = getYear() + 1;
	}

	if (getDay() <= monthDays[nextMon-1]){
		day = getDay();
	}
	else{
		day = monthDays[nextMon-1];
	}
	month = nextMon;
	year = nextYear;
}

void DateInterval::nextQuarter(int &day, int &month, int &year){
	day = 1;
	year = getYear();

	if (9 < getMonth()){		// im 4. Quartal -> 1. Quartal naechstes Jahr
		month = 1;
		year = getYear() + 1;
	}
	else if (6 < getMonth()){	// 3.-> 4. Quartal
		month = 10;
	}
	else if (3 < getMonth()){	// 2. -> 3. Quartal
		month = 7;
	}
	else {					// 1. -> 2. Quartal
		month = 4;
	}
}

void DateInterval::nextYear(int &day, int &month, int &year){
	day = getDay();
	month = getMonth();
	year = getYear() + 1;
}

void DateInterval::nextDecade(int &day, int &month, int &year){
	day = getDay();
	month = getMonth();
	year = getYear() + 10;
}
