// DateIntervalBasicTest.h  Datumsintervall-Berechner-Basis-Tests
//
// (c) 2003, oose.de GmbH
//////////////////////////////////////////////////////////////////////////

#ifndef DATE_INTERVAL_TESTS_H
#define DATE_INTERVAL_TESTS_H

#include <cppunit/extensions/HelperMacros.h>

class DateIntervalTest : public CppUnit::TestFixture{
	CPPUNIT_TEST_SUITE(DateIntervalTest);
	CPPUNIT_TEST(testDay);
	CPPUNIT_TEST(testWeek);
	CPPUNIT_TEST(testMonth);
	CPPUNIT_TEST(testQuarter);
	CPPUNIT_TEST(testYear);
	CPPUNIT_TEST(testDecade);
	CPPUNIT_TEST_SUITE_END();


private:
	int day, month, year;
	DateInterval* aDateInterval;

public:
//protected:
	void setUp(){
		aDateInterval = new DateInterval();
	}

	void tearDown(){
		delete aDateInterval;
	}

public:
	void testDay(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(7,10,1961));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(8 == day);
		CPPUNIT_ASSERT(10 == month);
		CPPUNIT_ASSERT(1961 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(28,2,1961));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(3 == month);
		CPPUNIT_ASSERT(1961 == year);
		CPPUNIT_ASSERT(true == aDateInterval->reset(31,12,2100));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(2101 == year);
		CPPUNIT_ASSERT(true == aDateInterval->reset(1,1,1800));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(2 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(1800 == year);
		CPPUNIT_ASSERT(true == aDateInterval->reset(28,2,2000));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(29 == day);
		CPPUNIT_ASSERT(2 == month);
		CPPUNIT_ASSERT(2000 == year);
		CPPUNIT_ASSERT(true == aDateInterval->reset(29,2,2000));
		aDateInterval->nextDay(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(3 == month);
		CPPUNIT_ASSERT(2000 == year);
	}

	void testWeek(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(7,10,1961));
		aDateInterval->nextWeek(day, month, year);
		CPPUNIT_ASSERT(14 == day);
		CPPUNIT_ASSERT(10 == month);
		CPPUNIT_ASSERT(1961 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(31,12,1961));
		aDateInterval->nextWeek(day, month, year);
		CPPUNIT_ASSERT(7 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(1962 == year);
	}

	void testMonth(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(7,10,1961));
		aDateInterval->nextMonth(day, month, year);
		CPPUNIT_ASSERT(7 == day);
		CPPUNIT_ASSERT(11 == month);
		CPPUNIT_ASSERT(1961 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(29,1,1996));
		aDateInterval->nextMonth(day, month, year);
		CPPUNIT_ASSERT(29 == day);
		CPPUNIT_ASSERT(2 == month);
		CPPUNIT_ASSERT(1996 == year);
	}

	void testQuarter(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(29,1,1996));
		aDateInterval->nextQuarter(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(4 == month);
		CPPUNIT_ASSERT(1996 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(1,3,1996));
		aDateInterval->nextQuarter(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(4 == month);
		CPPUNIT_ASSERT(1996 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(1,4,1996));
		aDateInterval->nextQuarter(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(7 == month);
		CPPUNIT_ASSERT(1996 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(1,10,1996));
		aDateInterval->nextQuarter(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(1997 == year);
	}

	void testYear(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(29,1,1996));
		aDateInterval->nextYear(day, month, year);
		CPPUNIT_ASSERT(29 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(1997 == year);

		CPPUNIT_ASSERT(true == aDateInterval->reset(29,2,1996));
		aDateInterval->nextYear(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(3 == month);
		CPPUNIT_ASSERT(1997 == year);
		

		CPPUNIT_ASSERT(true == aDateInterval->reset(31,12,2100));
		aDateInterval->nextYear(day, month, year);
		CPPUNIT_ASSERT(31 == day);
		CPPUNIT_ASSERT(12 == month);
		CPPUNIT_ASSERT(2101 == year);

		aDateInterval->nextYear(day, month, year);
		CPPUNIT_ASSERT(31 == day);
		CPPUNIT_ASSERT(12 == month);
		CPPUNIT_ASSERT(2101 == year);
		
	}

	void testDecade(){
		CPPUNIT_ASSERT(true == aDateInterval->reset(29,	01, 1996));
		aDateInterval->nextDecade(day, month, year);
		CPPUNIT_ASSERT(29 == day);
		CPPUNIT_ASSERT(1 == month);
		CPPUNIT_ASSERT(2006 == year);
		
		CPPUNIT_ASSERT(true == aDateInterval->reset(29,2,1996));
		aDateInterval->nextDecade(day, month, year);
		CPPUNIT_ASSERT(1 == day);
		CPPUNIT_ASSERT(3 == month);
		CPPUNIT_ASSERT(2006 == year);
	}

};

#endif
